@@ -25,6 +25,7 @@ import re
 import subprocess
 import sys
 from softwarecenter.utils import *
+from softwarecenter.enums import *
 
 from aptdaemon import client
 from aptdaemon import enums
@@ -35,15 +36,19 @@ from aptdaemon.gtkwidgets import AptMediumRequiredDialog, \
 from aptsources.sourceslist import SourceEntry
 from aptdaemon import policykit1
 
+# natty uses python-defer
 try:
-    from aptdaemon.defer import inline_callbacks, return_value
+    from defer import inline_callbacks, return_value
 except ImportError:
-    logging.getLogger("softwarecenter.backend").exception("aptdaemon import failed")
-    print 'Need the latest aptdaemon, try "sudo apt-add-repository ppa:software-store-developers/ppa" to get the PPA'
-    sys.exit(1)
+    # compat with maverick
+    try:
+        from aptdaemon.defer import inline_callbacks, return_value
+    except ImportError:
+        logging.getLogger("softwarecenter.backend").exception("aptdaemon import failed")
+        print 'Need the latest aptdaemon, try "sudo apt-add-repository ppa:software-store-developers/ppa" to get the PPA'
+        sys.exit(1)
 
 import gtk
-
 from softwarecenter.backend.transactionswatcher import TransactionsWatcher
 from softwarecenter.utils import get_http_proxy_string_from_gconf
 from softwarecenter.view import dialogs
@@ -82,7 +87,7 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
 
     __gsignals__ = {'transaction-started':(gobject.SIGNAL_RUN_FIRST,
                                             gobject.TYPE_NONE,
-                                            ()),
+                                            (str,str,str,str)),
                     # emits a TransactionFinished object
                     'transaction-finished':(gobject.SIGNAL_RUN_FIRST,
                                             gobject.TYPE_NONE,
@@ -123,20 +128,28 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
     def update_xapian_index(self):
         self._logger.debug("update_xapian_index")
         system_bus = dbus.SystemBus()
-        axi = dbus.Interface(
-            system_bus.get_object("org.debian.AptXapianIndex","/"),
-            "org.debian.AptXapianIndex")
+        # axi is optional, so just do nothing if its not installed
+        try:
+            axi = dbus.Interface(
+                system_bus.get_object("org.debian.AptXapianIndex","/"),
+                "org.debian.AptXapianIndex")
+        except dbus.DBusException as e:
+            self._logger.warning("axi can not be updated '%s'" % e)
+            return
         axi.connect_to_signal("UpdateFinished", self._axi_finished)
         # we don't really care for updates at this point
         #axi.connect_to_signal("UpdateProgress", progress)
-        # first arg is force, second update_only
-        axi.update_async(True, True)
+        try:
+            # first arg is force, second update_only
+            axi.update_async(True, False)
+        except:
+            self._logger.warning("could not update axi")
 
     @inline_callbacks
     def fix_broken_depends(self):
-        self.emit("transaction-started")
         try:
             trans = yield self.aptd_client.fix_broken_depends(defer=True)
+            self.emit("transaction-started", "", "", trans.tid, TRANSACTION_TYPE_REPAIR)
             yield self._run_transaction(trans, None, None, None)
         except Exception, error:
             self._on_trans_error(error)
@@ -145,10 +158,10 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
     @inline_callbacks
     def upgrade(self, pkgname, appname, iconname, addons_install=[], addons_remove=[], metadata=None):
         """ upgrade a single package """
-        self.emit("transaction-started")
         try:
             trans = yield self.aptd_client.upgrade_packages([pkgname],
                                                             defer=True)
+            self.emit("transaction-started", pkgname, appname, trans.tid, TRANSACTION_TYPE_UPGRADE)
             yield self._run_transaction(trans, pkgname, appname, iconname, metadata)
         except Exception, error:
             self._on_trans_error(error, pkgname)
@@ -182,10 +195,10 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
     @inline_callbacks
     def remove(self, pkgname, appname, iconname, addons_install=[], addons_remove=[], metadata=None):
         """ remove a single package """
-        self.emit("transaction-started")
         try:
             trans = yield self.aptd_client.remove_packages([pkgname],
                                                            defer=True)
+            self.emit("transaction-started", pkgname, appname, trans.tid, TRANSACTION_TYPE_REMOVE)
             yield self._run_transaction(trans, pkgname, appname, iconname, metadata)
         except Exception, error:
             self._on_trans_error(error, pkgname)
@@ -201,18 +214,27 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
             yield self.remove(pkgname, appname, iconname, metadata)
 
     @inline_callbacks
-    def install(self, pkgname, appname, iconname, filename=None, addons_install=[], addons_remove=[], metadata=None):
+    def install(self, pkgname, appname, iconname, filename=None, addons_install=[], addons_remove=[], metadata=None, force=False):
         """Install a single package from the archive
            If filename is given a local deb package is installed instead.
         """
-        self.emit("transaction-started")
         try:
             if filename:
-                trans = yield self.aptd_client.install_file(filename,
-                                                            defer=True)
+                # force means on lintian failure
+                trans = yield self.aptd_client.install_file(
+                    filename, force=force, defer=True)
+                self.emit("transaction-started", pkgname, appname, trans.tid, TRANSACTION_TYPE_INSTALL)
+                yield trans.set_meta_data(sc_filename=filename, defer=True)
             else:
-                trans = yield self.aptd_client.commit_packages([pkgname] + addons_install, [], addons_remove, [], [], defer=True)
-            yield self._run_transaction(trans, pkgname, appname, iconname, metadata)
+                install = [pkgname] + addons_install
+                remove = addons_remove
+                reinstall = remove = purge = upgrade =downgrade = []
+                trans = yield self.aptd_client.commit_packages(
+                    install, reinstall, remove, purge, upgrade, downgrade, 
+                    defer=True)
+                self.emit("transaction-started", pkgname, appname, trans.tid, TRANSACTION_TYPE_INSTALL)
+            yield self._run_transaction(
+                trans, pkgname, appname, iconname, metadata)
         except Exception, error:
             self._on_trans_error(error, pkgname)
 
@@ -229,9 +251,14 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
     @inline_callbacks
     def apply_changes(self, pkgname, appname, iconname, addons_install=[], addons_remove=[], metadata=None):
         """ install and remove add-ons """
-        self.emit("transaction-started")
         try:
-            trans = yield self.aptd_client.commit_packages(addons_install, [], addons_remove, [], [], defer=True)
+            install = addons_install
+            remove = addons_remove
+            reinstall = remove = purge = upgrade =downgrade = []
+            trans = yield self.aptd_client.commit_packages(
+                install, reinstall, remove, purge, upgrade, downgrade, 
+                defer=True)
+            self.emit("transaction-started", pkgname, appname, trans.tid, TRANSACTION_TYPE_APPLY)
             yield self._run_transaction(trans, pkgname, appname, iconname)
         except Exception, error:
             self._on_trans_error(error)
@@ -259,7 +286,10 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
     def enable_component(self, component):
         self._logger.debug("enable_component: %s" % component)
         try:
-            yield self.aptd_client.enable_distro_component(component, wait=True, defer=True)
+            trans = yield self.aptd_client.enable_distro_component(component)
+            # don't use self._run_transaction() here, to avoid sending uneeded
+            # signals
+            yield trans.run(defer=True)
         except Exception, error:
             self._on_trans_error(error, component)
             return_value(None)
@@ -280,7 +310,10 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
             yield self.add_sources_list_entry(entry, sourcepart)
             keyfile = channelfile.replace(".list",".key")
             if os.path.exists(keyfile):
-                yield self.aptd_client.add_vendor_key_from_file(keyfile, wait=True)
+                trans = yield self.aptd_client.add_vendor_key_from_file(keyfile, wait=True)
+                # don't use self._run_transaction() here, to avoid sending
+                # uneeded signals
+                yield trans.run(defer=True)
         yield self.reload(sourcepart)
 
     @inline_callbacks
@@ -346,7 +379,7 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
         and finally installing the specified application once the
         package list reload has completed.
         """
-        self.emit("transaction-started")
+        self.emit("transaction-started", app.pkgname, app.appname, "FIXME-NEED-ID-HERE", TRANSACTION_TYPE_INSTALL)
         self._logger.info("add_repo_add_key_and_install_app() '%s' '%s' '%s'"% (
                 # re.sub() out the password from the log
                 re.sub("deb https://.*@", "", deb_line),
@@ -402,10 +435,9 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
         # and then queue the install only when the reload finished
         # otherwise the daemon will fail because he does not know
         # the new package name yet
-        self._reload_signal_id = self.connect(
-            "reload-finished",
-            self._on_reload_for_add_repo_and_install_app_finished, 
-            trans_metadata, app)
+        self.connect("reload-finished",
+                     self._on_reload_for_add_repo_and_install_app_finished, 
+                     trans_metadata, app)
         # reload to ensure we have the new package data
         yield self.reload(sources_list=sources_list, metadata=trans_metadata)
 
@@ -450,8 +482,8 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
             result = False
 
         # disconnect again, this is only a one-time operation
-        self.handler_disconnect(self._reload_signal_id)
-        self._reload_signal_id = None
+        self.disconnect_by_func(
+            self._on_reload_for_add_repo_and_install_app_finished)
 
         # FIXME: this logic will *fail* if the sources.list of the user
         #        was broken before
@@ -547,38 +579,64 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
         except KeyError:
             pass
 
-    def _show_transaction_failed_dialog(self, trans, enum):
+    def _show_transaction_failed_dialog(self, trans, enum,
+                                        alternative_action=None):
         # daemon died are messages that result from broken
         # cancel handling in aptdaemon (LP: #440941)
         # FIXME: this is not a proper fix, just a workaround
         if trans.error_code == enums.ERROR_DAEMON_DIED:
             self._logger.warn("daemon dies, ignoring: %s %s" % (trans, enum))
             return
-        # hide any private ppa details in the error message since it may
-        # appear in the logs for LP bugs and potentially in screenshots as well
-        cleaned_error_details = obfuscate_private_ppa_details(trans.error_details)
         msg = "%s: %s\n%s\n\n%s" % (
             _("Error"),
             enums.get_error_string_from_enum(trans.error_code),
             enums.get_error_description_from_enum(trans.error_code),
-            cleaned_error_details)
+            trans.error_details)
         self._logger.error("error in _on_trans_finished '%s'" % msg)
+        # show dialog to the user and exit (no need to reopen the cache)
+        if not trans.error_code:
+            # sometimes aptdaemon doesn't return a value for error_code when the network
+            # connection has become unavailable; in that case, we will assume it's a
+            # failure during a package download because that is the only case where we
+            # see this happening - this avoids display of an empty error dialog and
+            # correctly prompts the user to check their network connection (see LP: #747172)
+            # FIXME: fix aptdaemon to return a valid error_code under all conditions
+            trans.error_code = enums.ERROR_PACKAGE_DOWNLOAD_FAILED
+        dialog_primary = enums.get_error_string_from_enum(trans.error_code)
+        dialog_secondary = enums.get_error_description_from_enum(trans.error_code)
+        dialog_details = trans.error_details
         # show dialog to the user and exit (no need to reopen
         # the cache)
-        dialogs.error(
-            None, 
-            enums.get_error_string_from_enum(trans.error_code),
-            enums.get_error_description_from_enum(trans.error_code),
-            cleaned_error_details)
+        return dialogs.error(None,
+                        enums.get_error_string_from_enum(trans.error_code),
+                        enums.get_error_description_from_enum(trans.error_code),
+                        trans.error_details,
+                        alternative_action)
 
     def _on_trans_finished(self, trans, enum):
         """callback when a aptdaemon transaction finished"""
         self._logger.debug("_on_transaction_finished: %s %s %s" % (
                 trans, enum, trans.meta_data))
 
+
         # show error
         if enum == enums.EXIT_FAILED:
-            if not "sc_add_repo_and_install_ignore_errors" in trans.meta_data:
+            # Handle invalid packages separately
+            if (trans.error and 
+                trans.error.code == enums.ERROR_INVALID_PACKAGE_FILE):
+                action = _("_Ignore and install")
+                res = self._show_transaction_failed_dialog(trans, enum, action)
+                if res == gtk.RESPONSE_YES:
+                    # Reinject the transaction
+                    meta_copy = trans.meta_data.copy()
+                    pkgname = meta_copy.pop("sc_pkgname")
+                    appname = meta_copy.pop("sc_appname", None)
+                    iconname = meta_copy.pop("sc_iconname", None)
+                    filename = meta_copy.pop("sc_filename")
+                    self.install(pkgname, appname, iconname, filename, [], [],
+                                 metadata=meta_copy, force=True)
+                    return
+            elif not "sc_add_repo_and_install_ignore_errors" in trans.meta_data:
                 self._show_transaction_failed_dialog(trans, enum)
 
         # send finished signal, use "" here instead of None, because
@@ -642,17 +700,21 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
                 # setup debconf only if we have a pkg
                 yield trans.set_debconf_frontend("gnome", defer=True)
                 trans.set_remove_obsoleted_depends(True, defer=True)
+                self._progress_signal = trans.connect("progress-changed", self._on_progress_changed)
+                self.pending_transactions[pkgname] = TransactionProgress(trans)
             # generic metadata
             if metadata:
                 yield trans.set_meta_data(defer=True, **metadata)
-            # do not set the http proxy by default
+            # FIXME: either use get_http_proxy_string_from_libproxy 
+            #        (with target url) or eliminate entirely
+            # do not set the http proxy by default (#628823)
             if os.environ.get("SOFTWARE_CENTER_USE_GCONF_PROXY"):
                 http_proxy = get_http_proxy_string_from_gconf()
                 if http_proxy:
                     trans.set_http_proxy(http_proxy, defer=True)
             yield trans.run(defer=True)
         except Exception, error:
-            self._on_trans_error(pkgname, error)
+            self._on_trans_error(error, pkgname)
             # on error we need to clean the pending purchases
             self._clean_pending_purchases(pkgname)
         # on success the pending purchase is cleaned when the package
@@ -669,6 +731,13 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
         # re-enable the action button again if anything went wrong
         result = TransactionFinishedResult(None, enums.EXIT_FAILED)
         result.pkgname = pkgname
+
+        # clean up pending transactions
+        try:
+            del self.pending_transactions[pkgname]
+        except:
+            pass
+
         self.emit("transaction-stopped", result)
         if isinstance(error, dbus.DBusException):
             name = error.get_dbus_name()
@@ -683,5 +752,8 @@ if __name__ == "__main__":
     #c = client.AptClient()
     #c.remove_packages(["4g8"], remove_unused_dependencies=True)
     backend = AptdaemonBackend()
-    backend.reload()
+    #backend.reload()
+    backend.enable_component("multiverse")
+
+    gtk.main()
 
