@@ -17,28 +17,21 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import os
-import xdg.BaseDirectory
+
+# pkgname of this app itself (used for "self-awareness", see spec)
+SOFTWARE_CENTER_PKGNAME = 'software-center'
 
 # buy-something base url
 #BUY_SOMETHING_HOST = "http://localhost:8000/"
 BUY_SOMETHING_HOST = os.environ.get("SOFTWARE_CENTER_BUY_HOST") or "https://software-center.ubuntu.com"
 BUY_SOMETHING_HOST_ANONYMOUS = os.environ.get("SOFTWARE_CENTER_BUY_HOST") or "http://software-center.ubuntu.com"
 
+# version of the database, every time something gets added (like 
+# terms for mime-type) increase this (but keep as a string!)
+DB_SCHEMA_VERSION = "2"
 
-# xapian pathes
-XAPIAN_BASE_PATH = "/var/cache/software-center"
-XAPIAN_BASE_PATH_SOFTWARE_CENTER_AGENT = os.path.join(
-    xdg.BaseDirectory.xdg_cache_home,
-    "software-center", 
-    "software-center-agent.db")
-
-# system pathes
-APP_INSTALL_PATH = "/usr/share/app-install"
-APP_INSTALL_DESKTOP_PATH = APP_INSTALL_PATH+"/desktop/"
-APP_INSTALL_CHANNELS_PATH = APP_INSTALL_PATH+"/channels/"
-ICON_PATH = APP_INSTALL_PATH+"/icons/"
-SOFTWARE_CENTER_BASE = "/usr/share/software-center"
-SOFTWARE_CENTER_PLUGIN_DIR = os.path.join(SOFTWARE_CENTER_BASE, "plugins")
+# the default limit for a search
+DEFAULT_SEARCH_LIMIT = 10000
 
 # the various "views" that the app has
 VIEW_PAGE_AVAILABLE = "view-page-available"
@@ -56,10 +49,26 @@ PERMANENT_VIEWS = [VIEW_PAGE_AVAILABLE,
                    VIEW_PAGE_CHANNEL,
                    VIEW_PAGE_HISTORY,
                   ]
+                  
+# define ID values for the various buttons found in the navigation bar
+NAV_BUTTON_ID_CATEGORY = "category"
+NAV_BUTTON_ID_LIST     = "list"
+NAV_BUTTON_ID_SUBCAT   = "subcat"
+NAV_BUTTON_ID_DETAILS  = "details"
+NAV_BUTTON_ID_SEARCH   = "search"
+NAV_BUTTON_ID_PURCHASE = "purchase"
+NAV_BUTTON_ID_PREV_PURCHASES = "prev-purchases"
+
+# define ID values for the action bar buttons
+ACTION_BUTTON_ID_INSTALL = "install"
+ACTION_BUTTON_ADD_TO_LAUNCHER = "add_to_launcher"
+ACTION_BUTTON_CANCEL_ADD_TO_LAUNCHER = "cancel_add_to_launcher"
 
 # icons
 MISSING_APP_ICON = "applications-other"
 MISSING_PKG_ICON = "dialog-question"
+APP_ICON_SIZE = 48
+GENERIC_MISSING_IMAGE = "gtk-missing-image"
 
 # sorting
 (SORT_UNSORTED,
@@ -88,7 +97,7 @@ XAPIAN_VALUE_SCREENSHOT_URL = 185
 XAPIAN_VALUE_ICON_NEEDS_DOWNLOAD = 186
 XAPIAN_VALUE_THUMBNAIL_URL = 187
 XAPIAN_VALUE_SC_DESCRIPTION = 188
-XAPIAN_VALUE_ICON_URL = 190
+XAPIAN_VALUE_APPNAME_UNTRANSLATED = 189
 
 # fake channels
 PURCHASED_NEEDS_REINSTALL_MAGIC_CHANNEL_NAME = "for-pay-needs-reinstall"
@@ -133,14 +142,20 @@ APP_ACTION_REMOVE = "remove"
 APP_ACTION_UPGRADE = "upgrade"
 APP_ACTION_APPLY = "apply_changes"
 
-from version import *
+# transaction types
+TRANSACTION_TYPE_INSTALL = "install"
+TRANSACTION_TYPE_REMOVE = "remove"
+TRANSACTION_TYPE_UPGRADE = "upgrade"
+TRANSACTION_TYPE_APPLY = "apply_changes"
+TRANSACTION_TYPE_REPAIR = "repair_dependencies"
+
+# delimiter for directory path separator in app-install
+APP_INSTALL_PATH_DELIMITER = "__"
+
+from version import VERSION, DISTRO, RELEASE, CODENAME
 USER_AGENT="Software Center/%s (N;) %s/%s (%s)" % (VERSION, 
                                                    DISTRO, 
                                                    RELEASE,
                                                    CODENAME)
                                                    
-# FIXME: use relative paths here
-INSTALLED_ICON = "/usr/share/software-center/icons/software-center-installed.png"
-IMAGE_LOADING = "/usr/share/icons/hicolor/32x32/animations/softwarecenter-loading.gif"
-IMAGE_LOADING_INSTALLED = "/usr/share/icons/hicolor/32x32/animations/softwarecenter-loading-installed.gif"
 
