@@ -19,15 +19,18 @@
 
 import logging
 import gtk
+import dialogs
+
 import urllib
 import gobject
 
 from softwarecenter.db.application import AppDetails
+from softwarecenter.db.reviews import get_review_loader
 from softwarecenter.backend import get_install_backend
 from softwarecenter.enums import *
-from softwarecenter.utils import get_current_arch, get_default_language
+from softwarecenter.utils import get_current_arch, get_parent_xid, get_language
 
-from purchasedialog import PurchaseDialog
+LOG=logging.getLogger(__name__)
 
 class AppDetailsViewBase(object):
 
@@ -37,25 +40,28 @@ class AppDetailsViewBase(object):
                                         (gobject.TYPE_PYOBJECT, 
                                          gobject.TYPE_PYOBJECT, 
                                          gobject.TYPE_PYOBJECT, 
-                                         str),
-                                       ),
+                                         str,)),
+         "purchase-requested" : (gobject.SIGNAL_RUN_LAST,
+                                 gobject.TYPE_NONE,
+                                 (gobject.TYPE_PYOBJECT,
+                                  str,)),
     }
 
-    def __init__(self, db, distro, icons, cache, history, datadir):
+    def __init__(self, db, distro, icons, cache, datadir):
         self.db = db
         self.distro = distro
         self.icons = icons
         self.cache = cache
         self.cache.connect("cache-ready", self._on_cache_ready)
-        self.history = history
         self.datadir = datadir
         self.app = None
         self.appdetails = None
         self.addons_to_install = []
         self.addons_to_remove = []
+        # reviews
+        self.review_loader = get_review_loader(self.cache, self.db)
         # aptdaemon
         self.backend = get_install_backend()
-        self._logger = logging.getLogger(__name__)
         
     def _draw(self):
         """ draw the current app into the window, maybe the function
@@ -72,9 +78,56 @@ class AppDetailsViewBase(object):
         #print "AppDetailsViewWebkit:"
         #print self.appdetails
         self._draw()
+        self._check_for_reviews()
         self.emit("selected", self.app)
     def refresh_app(self):
         self.show_app(self.app)
+
+    # common code
+    def _review_write_new(self):
+        if (not self.app or
+            not self.app.pkgname in self.cache or
+            not self.cache[self.app.pkgname].candidate):
+            dialogs.error(None, 
+                          _("Version unknown"),
+                          _("The version of the application can not "
+                            "be detected. Entering a review is not "
+                            "possible."))
+            return
+        # gather data
+        pkg = self.cache[self.app.pkgname]
+        version = pkg.candidate.version
+        origin = self.cache.get_origin(self.app.pkgname)
+
+        # FIXME: probably want to not display the ui if we can't review it
+        if not origin:
+            dialogs.error(None, 
+                        _("Origin unknown"),
+                        _("The origin of the application can not "
+                          "be detected. Entering a review is not "
+                          "possible."))
+            return
+
+        if pkg.installed:
+            version = pkg.installed.version
+        # call the loader to do call out the right helper and collect the result
+        parent_xid = get_parent_xid(self)
+        self.review_loader.spawn_write_new_review_ui(
+            self.app, version, self.appdetails.icon, origin,
+            parent_xid, self.datadir,
+            self._reviews_ready_callback)
+                         
+    def _review_report_abuse(self, review_id):
+        parent_xid = get_parent_xid(self)
+        self.review_loader.spawn_report_abuse_ui(
+            review_id, parent_xid, self.datadir, self._reviews_ready_callback)
+
+    def _review_submit_usefulness(self, review_id, is_useful):
+        parent_xid = get_parent_xid(self)
+        self.review_loader.spawn_submit_usefulness_ui(
+            review_id, is_useful, parent_xid, self.datadir,
+            self._reviews_ready_callback)
+
     # public interface
     def reload(self):
         """ reload the package cache, this goes straight to the backend """
@@ -94,26 +147,18 @@ class AppDetailsViewBase(object):
 
     def buy_app(self):
         """ initiate the purchase transaction """
-        lang = get_default_language()
-        url = self.distro.PURCHASE_APP_URL % (lang, urllib.urlencode({
+        lang = get_language()
+        distro = self.distro.get_codename()
+        url = self.distro.PURCHASE_APP_URL % (lang, distro, urllib.urlencode({
                     'archive_id' : self.appdetails.ppaname, 
                     'arch' : get_current_arch() ,
                     }))
-        appdetails = self.app.get_details(self.db)
-        self.purchase_dialog = PurchaseDialog(url=url, 
-                                              app=self.app,
-                                              iconname=appdetails.icon)
-        res = self.purchase_dialog.run()
-        self.purchase_dialog.destroy()
-        del self.purchase_dialog
-        # re-init view if user canceled, otherwise the transactions 
-        # will finish it after some time
-        if res != gtk.RESPONSE_OK:
-            self.show_app(self.app)
+        
+        self.emit("purchase-requested", self.app, url)
 
     def reinstall_purchased(self):
         """ reinstall a purchased app """
-        self._logger.debug("reinstall_purchased %s" % self.app)
+        LOG.debug("reinstall_purchased %s" % self.app)
         appdetails = self.app.get_details(self.db)
         iconname = appdetails.icon
         deb_line = appdetails.deb_line
@@ -127,5 +172,7 @@ class AppDetailsViewBase(object):
     def _on_cache_ready(self, cache):
         # re-show the application if the cache changes, it may affect the
         # current application
-        self._logger.debug("on_cache_ready")
+        LOG.debug("on_cache_ready")
         self.show_app(self.app)
+
+
